package ffp_examples;

/*
 *  blockfilter.java
 * 	This program demonstrates the mipmap-like effect of block filtering.
 *  The top plane uses block filter, the bottom plane does not. Notice
 *  that the texels are identical. The block filtered plane preserves
 *  the lines in the texture by using the block average of the texture
 *  as the texel gets further away.
 *  Hold T to increase the distance at which block filtering begins and
 *  hold G to decrease that distance.
 */

public class blockfilter extends DemoBase {

	public static void main(String[] args) {
		new blockfilter().init(EUT_RGB);
	}

	static int tex_width = 256;
	static int tex_height = 256;
	static int[] tex = new int[tex_width * tex_height];
	static float dist = 0.6f;

	static void gen_tex() {
		int txW = tex_width, txH = tex_height;
		for (int x = 0; x < txW; x++) {
			for (int y = 0; y < txH; y++) {
				boolean bb = (x < 8 || y < 8 || (x + 8) >= txW || (y + 8) >= txH);
				int c = ((bb ? 1 : 0)) * 255;
				tex[x + y * txW] = c << 16 | c << 8 | c;
			}
		}
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(60.0f, (float) w / (float) h, 1f, 1000.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_FLAT);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		gen_tex();
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_NEAREST);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_BLOCK_FILTER, FFP_FALSE);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, tex, tex_width, tex_height);
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 1);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_NEAREST);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_BLOCK_FILTER, FFP_TRUE);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, tex, tex_width, tex_height);
		ffp.ffpTexBlockFilterStart(dist);
		ffp.ffpEnable(FFP_TEXTURE_2D);
	}

	@Override
	public void dbUpdate() {
		if (keyHeld(KEY_T)) {
			dist += 0.01f;
			if (dist > 0.999f) {
				dist = 0.999f;
			}
			ffp.ffpTexBlockFilterStart(dist);
		}
		if (keyHeld(KEY_G)) {
			dist -= 0.01f;
			if (dist < 0) {
				dist = 0.0f;
			}
			ffp.ffpTexBlockFilterStart(dist);
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		int tex_scale = 40;
		ffp.ffpPushMatrix();
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpBegin(FFP_QUADS);

		ffp.ffpTexCoord2f(0.0f, 0.0f);
		ffp.ffpVertex3f(-8.0f, 1.0f, 0.0f);

		ffp.ffpTexCoord2f(tex_scale, 0.0f);
		ffp.ffpVertex3f(-8.0f, 1.0f, -100.0f);

		ffp.ffpTexCoord2f(tex_scale, tex_scale);
		ffp.ffpVertex3f(8.0f, 1.0f, -100.0f);

		ffp.ffpTexCoord2f(0.0f, tex_scale);
		ffp.ffpVertex3f(8.0f, 1.0f, 0.0f);

		ffp.ffpEnd();
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 1);
		ffp.ffpBegin(FFP_QUADS);

		ffp.ffpTexCoord2f(0.0f, 0.0f);
		ffp.ffpVertex3f(-8.0f, -1.0f, 0.0f);

		ffp.ffpTexCoord2f(tex_scale, 0.0f);
		ffp.ffpVertex3f(-8.0f, -1.0f, -100.0f);

		ffp.ffpTexCoord2f(tex_scale, tex_scale);
		ffp.ffpVertex3f(8.0f, -1.0f, -100.0f);

		ffp.ffpTexCoord2f(0.0f, tex_scale);
		ffp.ffpVertex3f(8.0f, -1.0f, 0.0f);

		ffp.ffpEnd();
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
