package ffp_examples;

/*
 *  chaos.java
 *  Using the growth formula x[n+1] = L * x[n] * (1-x[n])
 *  We can see Feigenbaum's constant's use in chaos and bifurcation.
 */
public class chaos extends DemoBase {

	public static void main(String[] args) {
		new chaos().init(EUT_RGB);
	}

	static final float X_MAX = 50;
	static final float LAMBDA_START = 0;
	static final float LAMBDA_END = 4;
	static final float Y_INIT = 0.5f;

	static float zoom = 0.0f;
	static float x_shift = 0.0f;
	static float y_shift = 0.0f;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection();
	}

	private static void create_projection() {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		ffp.ffpOrtho(-zoom + x_shift, LAMBDA_END + zoom + x_shift, -zoom + y_shift, 1.0f + zoom + y_shift, -5.0f, 5.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
	}

	@Override
	public void dbUpdate() {
		if (keyHeld(KEY_W)) {
			zoom -= 0.1;
			create_projection();
		}
		if (keyHeld(KEY_S)) {
			zoom += 0.1;
			create_projection();
		}
		if (keyHeld(KEY_RIGHT)) {
			x_shift += 0.1;
			create_projection();
		}
		if (keyHeld(KEY_LEFT)) {
			x_shift -= 0.1;
			create_projection();
		}
		if (keyHeld(KEY_UP)) {
			y_shift += 0.1;
			create_projection();
		}
		if (keyHeld(KEY_DOWN)) {
			y_shift -= 0.1;
			create_projection();
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER);
		ffp.ffpColor3f(1, 0, 0);
		ffp.ffpBegin(FFP_POINTS);

		for (float L = LAMBDA_START; L < LAMBDA_END; L += 0.003) {
			float y = Y_INIT;
			for (float i = 0; i < X_MAX; i++) {
				y = L * y * (1 - y);
				ffp.ffpVertex3f(L, y, -3.0f);
			}
		}
		ffp.ffpEnd();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
