package ffp_examples;

/*
 *  cloth.java
 */
public class cloth extends DemoBase {

	public static void main(String[] args) {
		new cloth().init(EUT_RGB);
	}

	static final float PI = 3.14159f;

	static float[][] x = new float[64][64];
	static float[][] y = new float[64][64];
	static float[][] z = new float[64][64];
	static float[][] nx = new float[64][64];
	static float[][] ny = new float[64][64];
	static float[][] nz = new float[64][64];
	static int spheres = 5;
	static float[] ssx = new float[spheres];
	static float[] ssy = new float[spheres];
	static float[] ssz = new float[spheres];
	static float[] ssr = new float[spheres];
	static float[] sx = new float[spheres];
	static float[] sy = new float[spheres];
	static float[] sz = new float[spheres];
	static float[] sr = new float[spheres];

	static float[][] dist = new float[spheres][spheres];

	static float r1 = 0;
	static float r2 = 0;
	static float a = 0;

	static float[][] sphere_x = new float[16][16];
	static float[][] sphere_y = new float[16][16];
	static float[][] sphere_z = new float[16][16];

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(60.0f, (float) w / (float) h, 1.0f, 10.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_CULL_FACE);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		Initialise();
	}

	void Sphere(double sx, double sy, double sz, double sr) {
		int i, j;
		double x, y, z;

		for (j = 0; j < 16; j++) {
			for (i = 0; i < 16; i++) {
				x = sr * sin(i * PI / 16);
				y = sr * cos(i * PI / 16);
				z = 0;

				//  Rotate about y axis

				double nx = z * sin(j * PI / 8) + x * cos(j * PI / 8);
				double ny = y;
				double nz = z * cos(j * PI / 8) - x * sin(j * PI / 8);

				sphere_x[j][i] = (float) (sx + nx);
				sphere_y[j][i] = (float) (sy + ny);
				sphere_z[j][i] = (float) (sz + nz);
			}
		}
		float r, g, b;
		int ii, jj;

		for (j = 0; j < 16; j++) {
			for (i = 0; i < 16; i++) {

				ii = (i + 1) & 15;
				jj = (j + 1) & 15;

				if (((i ^ j) & 1) == 0) {
					r = 1.0f;
					g = 1.0f;
					b = 1.0f;
				} else {
					r = 1.0f;
					g = 0.0f;
					b = 0.0f;
				}

				ffp.ffpBegin(FFP_TRIANGLE_STRIP);
				ffp.ffpColor3f(r, g, b);
				ffp.ffpVertex3f(sphere_x[j][i], sphere_y[j][i], sphere_z[j][i]);
				ffp.ffpVertex3f(sphere_x[j][ii], sphere_y[j][ii], sphere_z[j][ii]);
				ffp.ffpVertex3f(sphere_x[jj][i], sphere_y[jj][i], sphere_z[jj][i]);
				ffp.ffpVertex3f(sphere_x[jj][ii], sphere_y[jj][ii], sphere_z[jj][ii]);
				ffp.ffpEnd();

			}
		}
	}

	void Initialise() {
		int i, j;

		for (j = 0; j < 32; j++) {
			for (i = 0; i < 32; i++) {
				x[j][i] = (i - 16) * 0.1f;
				y[j][i] = 1.5f;
				z[j][i] = (j - 16) * 0.1f;
			}
		}
		for (j = 0; j < 5; j++) {
			for (i = 0; i < 5; i++) {
				dist[j][i] = sqrt((2 - i) * (2 - i) + (2 - j) * (2 - j));
			}
		}
		ssx[0] = 0.9f;
		ssy[0] = -0.7f;
		ssz[0] = 0.9f;
		ssr[0] = 0.5f;
		ssx[1] = 0.2f;
		ssy[1] = -0.4f;
		ssz[1] = -0.8f;
		ssr[1] = 0.5f;
		ssx[2] = -0.4f;
		ssy[2] = -0.3f;
		ssz[2] = 0.6f;
		ssr[2] = 0.5f;
		ssx[3] = -0.3f;
		ssy[3] = -1.7f;
		ssz[3] = -0.7f;
		ssr[3] = 0.5f;
		ssx[4] = -0.6f;
		ssy[4] = -1.2f;
		ssz[4] = -0.3f;
		ssr[4] = 0.5f;
	}

	@Override
	public void dbUpdate() {
		int i, j, ii, jj, k, kk;

		float d, dd;

		//  Move spheres

		for (i = 0; i < 5; i++) {
			sx[i] = ssx[i];
			sy[i] = ssy[i];
			sz[i] = ssz[i];
			sr[i] = ssr[i];
		}

		a++;

		sy[0] = (float) (ssy[0] - 1.16 * cos(-a * PI * 0.0007722 - 0.5947));
		sy[1] = (float) (ssy[1] - 1.67 * cos(-a * PI * 0.0006224 + 0.1344));
		sy[2] = (float) (ssy[2] - 1.42 * sin(-a * PI * 0.0004629 + 0.6143));
		sy[3] = (float) (ssy[3] + 1.55 * cos(-a * PI * 0.0003726 - 0.8941));
		sy[4] = (float) (ssy[4] - 1.71 * cos(-a * PI * 0.0001722 - 0.7228));

		//  Move cloth

		float mx, my, mz;
		float vx, vy, vz;

		for (k = 0; k < 15; k++) {
			for (j = 0; j < 32; j++) {
				for (i = 0; i < 32; i++) {
					mx = 0;
					my = -0.00098f;
					mz = 0;

					for (jj = j - 1; jj <= j + 1; jj++)
						if (jj >= 0 && jj < 32) {
							for (ii = i - 1; ii <= i + 1; ii++)
								if (ii >= 0 && ii < 32)
									if (ii != i || jj != j) {
										vx = x[jj][ii] - x[j][i];
										vy = y[jj][ii] - y[j][i];
										vz = z[jj][ii] - z[j][i];

										d = vx * vx + vy * vy + vz * vz;
										dd = (float) (0.03 * (sqrt(d) - 0.1 * dist[2 + jj - j][2 + ii - i]) / (d));

										mx += vx * dd;
										my += vy * dd;
										mz += vz * dd;
									}
						}

					nx[j][i] = x[j][i] + mx;
					ny[j][i] = y[j][i] + my;
					nz[j][i] = z[j][i] + mz;

					//  Floor collision

					if (ny[j][i] < -2.9f) {
						ny[j][i] = -2.9f;
					}
					//  Sphere collision

					for (kk = 0; kk < spheres; kk++) {
						d = (nx[j][i] - sx[kk]) * (nx[j][i] - sx[kk]) + (ny[j][i] - sy[kk]) * (ny[j][i] - sy[kk]) + (nz[j][i] - sz[kk]) * (nz[j][i] - sz[kk]);
						if (d < sr[kk] * sr[kk]) {
							d = sr[kk] / sqrt(d);
							nx[j][i] = (nx[j][i] - sx[kk]) * d + sx[kk];
							ny[j][i] = (ny[j][i] - sy[kk]) * d + sy[kk];
							nz[j][i] = (nz[j][i] - sz[kk]) * d + sz[kk];
						}
					}
				}
			}

			for (j = 0; j < 32; j++)
				for (i = 0; i < 32; i++) {
					x[j][i] = nx[j][i];
					y[j][i] = ny[j][i];
					z[j][i] = nz[j][i];
				}

		}

	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -5);

		ffp.ffpRotatef(-r2 * 0.2f, 1, 0, 0);
		ffp.ffpRotatef(-r1 * 0.2f, 0, 1, 0);

		GameDraw();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

	void GameDraw() {
		int i, j, k;
		float r, g, b;

		//  Draw spheres
		ffp.ffpCullFace(FFP_BACK);

		for (k = 0; k < spheres; k++) {
			Sphere(sx[k], sy[k], sz[k], sr[k] - 0.01);
		}
		ffp.ffpCullFace(FFP_FRONT);

		//  Draw floor

		for (j = 0; j < 8; j++) {
			for (i = 0; i < 8; i++) {
				if (((i ^ j) & 1) == 0) {
					r = 0.9f;
					g = 0.9f;
					b = 0.9f;
				} else {
					r = 0.1f;
					g = 0.1f;
					b = 0.1f;
				}

				ffp.ffpBegin(FFP_TRIANGLE_STRIP);
				ffp.ffpColor3f(r, g, b);
				ffp.ffpVertex3f(0.5f * (i - 4), -3, 0.5f * (j - 4));
				ffp.ffpVertex3f(0.5f * (i - 3), -3, 0.5f * (j - 4));
				ffp.ffpVertex3f(0.5f * (i - 4), -3, 0.5f * (j - 3));
				ffp.ffpVertex3f(0.5f * (i - 3), -3, 0.5f * (j - 3));
				ffp.ffpEnd();

			}
		}

		//  Draw cloth
		ffp.ffpDisable(FFP_CULL_FACE);

		for (j = 0; j < 31; j++) {
			for (i = 0; i < 31; i++) {

				if (((i ^ j) & 1) == 0) {
					r = 1.0f;
					g = 1.0f;
					b = 0.0f;
				} else {
					r = 0.0f;
					g = 0.0f;
					b = 1.0f;
				}

				ffp.ffpBegin(FFP_TRIANGLE_STRIP);
				ffp.ffpColor3f(r, g, b);
				ffp.ffpVertex3f(x[j][i], y[j][i], z[j][i]);
				ffp.ffpColor3f(r, g, b);
				ffp.ffpVertex3f(x[j][i + 1], y[j][i + 1], z[j][i + 1]);
				ffp.ffpColor3f(r, g, b);
				ffp.ffpVertex3f(x[j + 1][i], y[j + 1][i], z[j + 1][i]);
				ffp.ffpColor3f(r, g, b);
				ffp.ffpVertex3f(x[j + 1][i + 1], y[j + 1][i + 1], z[j + 1][i + 1]);
				ffp.ffpEnd();

			}
		}
		ffp.ffpEnable(FFP_CULL_FACE);
	}

}
