package ffp_examples;

/*
 * colorcube.java
 * This program shows a smooth shaded color cube.
 */
public class colorcube extends DemoBase {

	public static void main(String[] args) {
		new colorcube().init(EUT_RGB);
	}

	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 4;
	static int xold, yold;

	static float size = 0.5f;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(45, w / (float) h, 1f, 150f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_DEPTH_TEST);
	}

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}
		xold = x;
		yold = y;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);

		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -translate_z);
		ffp.ffpRotatef(rotate_y, 1, 0, 0);
		ffp.ffpRotatef(rotate_x, 0, 1, 0);
		// Front Face
		ffp.ffpBegin(FFP_QUADS);
		ffp.ffpColor3f(0, 0, 1);
		ffp.ffpVertex3f(-size, -size, size);
		ffp.ffpColor3f(1, 0, 1);
		ffp.ffpVertex3f(size, -size, size);
		ffp.ffpColor3f(1, 1, 1);
		ffp.ffpVertex3f(size, size, size);
		ffp.ffpColor3f(0, 1, 1);
		ffp.ffpVertex3f(-size, size, size);
		// Back Face
		ffp.ffpColor3f(0, 0, 0);
		ffp.ffpVertex3f(-size, -size, -size);
		ffp.ffpColor3f(0, 1, 0);
		ffp.ffpVertex3f(-size, size, -size);
		ffp.ffpColor3f(1, 1, 0);
		ffp.ffpVertex3f(size, size, -size);
		ffp.ffpColor3f(1, 0, 0);
		ffp.ffpVertex3f(size, -size, -size);
		// Top Face
		ffp.ffpColor3f(0, 1, 0);
		ffp.ffpVertex3f(-size, size, -size);
		ffp.ffpColor3f(0, 1, 1);
		ffp.ffpVertex3f(-size, size, size);
		ffp.ffpColor3f(1, 1, 1);
		ffp.ffpVertex3f(size, size, size);
		ffp.ffpColor3f(1, 1, 0);
		ffp.ffpVertex3f(size, size, -size);
		// Bottom Face
		ffp.ffpColor3f(0, 0, 0);
		ffp.ffpVertex3f(-size, -size, -size);
		ffp.ffpColor3f(1, 0, 0);
		ffp.ffpVertex3f(size, -size, -size);
		ffp.ffpColor3f(1, 0, 1);
		ffp.ffpVertex3f(size, -size, size);
		ffp.ffpColor3f(0, 0, 1);
		ffp.ffpVertex3f(-size, -size, size);
		// Right face
		ffp.ffpColor3f(1, 0, 0);
		ffp.ffpVertex3f(size, -size, -size);
		ffp.ffpColor3f(1, 1, 0);
		ffp.ffpVertex3f(size, size, -size);
		ffp.ffpColor3f(1, 1, 1);
		ffp.ffpVertex3f(size, size, size);
		ffp.ffpColor3f(1, 0, 1);
		ffp.ffpVertex3f(size, -size, size);
		// Left Face
		ffp.ffpColor3f(0, 0, 0);
		ffp.ffpVertex3f(-size, -size, -size);
		ffp.ffpColor3f(0, 0, 1);
		ffp.ffpVertex3f(-size, -size, size);
		ffp.ffpColor3f(0, 1, 1);
		ffp.ffpVertex3f(-size, size, size);
		ffp.ffpColor3f(0, 1, 0);
		ffp.ffpVertex3f(-size, size, -size);
		ffp.ffpEnd();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
