package ffp_examples;

/*
 *  gears.java
 *  A program with a function to draw a gear and transform them.
 */
public class gears extends DemoBase {

	public static void main(String[] args) {
		new gears().init(EUT_RGB);
	}

	static float view_rotx = 20.0f, view_roty = 30.0f, view_rotz = 0.0f;
	static float angle = 0.0f;
	static final float M_PI = 3.14159265f;
	static float pos[] = { 5.0f, 5.0f, 10.0f, 0.0f };
	static float red[] = { 0.8f, 0.0f, 0.0f, 1.0f };
	static float green[] = { 0.0f, 0.8f, 0.0f, 1.0f };
	static float blue[] = { 0.0f, 0.0f, 0.8f, 1.0f };

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		float asp = (float) h / (float) w;
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		ffp.ffpFrustum(-1.0f, 1.0f, -asp, asp, 5.0f, 60.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -40.0f);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_NORMALIZE);

		ffp.ffpLightfv(0, FFP_POSITION, pos);
	}

	@Override
	public void dbUpdate() {
		if (keyHeld(KEY_LEFT)) {
			view_roty += 5.0f;
		}
		if (keyHeld(KEY_RIGHT)) {
			view_roty -= 5.0f;
		}
		if (keyHeld(KEY_UP)) {
			view_rotx += 5.0f;
		}
		if (keyHeld(KEY_DOWN)) {
			view_rotx -= 5.0f;
		}
		angle++;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);

		ffp.ffpPushMatrix();
		ffp.ffpRotatef(view_rotx, 1.0f, 0.0f, 0.0f);
		ffp.ffpRotatef(view_roty, 0.0f, 1.0f, 0.0f);
		ffp.ffpRotatef(view_rotz, 0.0f, 0.0f, 1.0f);

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-3.0f, -2.0f, 0.0f);
		ffp.ffpRotatef(angle, 0.0f, 0.0f, 1.0f);
		ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, red);
		gear(1.0f, 4.0f, 1.0f, 20, 0.7f);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(3.1f, -2.0f, 0.0f);
		ffp.ffpRotatef(-2.0f * angle - 9.0f, 0.0f, 0.0f, 1.0f);
		ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, green);
		gear(0.5f, 2.0f, 2.0f, 10, 0.7f);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-3.1f, 4.2f, 0.0f);
		ffp.ffpRotatef(-2.0f * angle - 25.0f, 0.0f, 0.0f, 1.0f);
		ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, blue);
		gear(1.3f, 2.0f, 0.5f, 10, 0.7f);
		ffp.ffpPopMatrix();

		ffp.ffpPopMatrix();

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

	/*
	 *
	 *  Draw a gear wheel.
	 *  Input:  inner_radius - radius of hole at center
	 *          outer_radius - radius at center of teeth
	 *          width - width of gear
	 *          teeth - number of teeth
	 *          tooth_depth - depth of tooth
	 */
	static void gear(float inner_radius, float outer_radius, float width, int teeth, float tooth_depth) {
		int i;
		float r0, r1, r2;
		float angle, da;
		float u, v, len;

		r0 = inner_radius;
		r1 = outer_radius - tooth_depth / 2.0f;
		r2 = outer_radius + tooth_depth / 2.0f;

		da = 2.0f * 3 / teeth / 4.0f;

		ffp.ffpShadeModel(FFP_FLAT);

		/* draw front face */
		ffp.ffpBegin(FFP_QUAD_STRIP);
		ffp.ffpNormal3f(0.0f, 0.0f, 1.0f);
		for (i = 0; i <= teeth; i++) {
			angle = i * 2.0f * M_PI / teeth;
			ffp.ffpVertex3f(r0 * cos(angle), r0 * sin(angle), width * 0.5f);
			ffp.ffpVertex3f(r1 * cos(angle), r1 * sin(angle), width * 0.5f);
			if (i < teeth) {
				ffp.ffpVertex3f(r0 * cos(angle), r0 * sin(angle), width * 0.5f);
				ffp.ffpVertex3f(r1 * cos(angle + 3 * da), r1 * sin(angle + 3 * da), width * 0.5f);
			}
		}
		ffp.ffpEnd();

		/* draw front sides of teeth */
		ffp.ffpBegin(FFP_QUADS);
		da = 2.0f * M_PI / teeth / 4.0f;
		for (i = 0; i < teeth; i++) {
			angle = i * 2.0f * M_PI / teeth;

			ffp.ffpVertex3f(r1 * cos(angle), r1 * sin(angle), width * 0.5f);
			ffp.ffpVertex3f(r2 * cos(angle + da), r2 * sin(angle + da), width * 0.5f);
			ffp.ffpVertex3f(r2 * cos(angle + 2 * da), r2 * sin(angle + 2 * da), width * 0.5f);
			ffp.ffpVertex3f(r1 * cos(angle + 3 * da), r1 * sin(angle + 3 * da), width * 0.5f);
		}
		ffp.ffpEnd();

		/* draw back face */
		ffp.ffpBegin(FFP_QUAD_STRIP);
		ffp.ffpNormal3f(0.0f, 0.0f, -1.0f);

		for (i = 0; i <= teeth; i++) {
			angle = i * 2.0f * M_PI / teeth;
			ffp.ffpVertex3f(r1 * cos(angle), r1 * sin(angle), -width * 0.5f);
			ffp.ffpVertex3f(r0 * cos(angle), r0 * sin(angle), -width * 0.5f);
			if (i < teeth) {
				ffp.ffpVertex3f(r1 * cos(angle + 3 * da), r1 * sin(angle + 3 * da), -width * 0.5f);
				ffp.ffpVertex3f(r0 * cos(angle), r0 * sin(angle), -width * 0.5f);
			}
		}
		ffp.ffpEnd();

		/* draw back sides of teeth */
		ffp.ffpBegin(FFP_QUADS);
		da = 2.0f * M_PI / teeth / 4.0f;
		for (i = 0; i < teeth; i++) {
			angle = i * 2.0f * M_PI / teeth;

			ffp.ffpVertex3f(r1 * cos(angle + 3 * da), r1 * sin(angle + 3 * da), -width * 0.5f);
			ffp.ffpVertex3f(r2 * cos(angle + 2 * da), r2 * sin(angle + 2 * da), -width * 0.5f);
			ffp.ffpVertex3f(r2 * cos(angle + da), r2 * sin(angle + da), -width * 0.5f);
			ffp.ffpVertex3f(r1 * cos(angle), r1 * sin(angle), -width * 0.5f);
		}
		ffp.ffpEnd();

		/* draw outward faces of teeth */
		ffp.ffpBegin(FFP_QUAD_STRIP);
		for (i = 0; i < teeth; i++) {
			angle = i * 2.0f * M_PI / teeth;

			ffp.ffpVertex3f(r1 * cos(angle), r1 * sin(angle), width * 0.5f);
			ffp.ffpVertex3f(r1 * cos(angle), r1 * sin(angle), -width * 0.5f);
			u = r2 * cos(angle + da) - r1 * cos(angle);
			v = r2 * sin(angle + da) - r1 * sin(angle);
			len = sqrt(u * u + v * v);
			u /= len;
			v /= len;
			ffp.ffpNormal3f(v, -u, 0.0f);
			ffp.ffpVertex3f(r2 * cos(angle + da), r2 * sin(angle + da), width * 0.5f);
			ffp.ffpVertex3f(r2 * cos(angle + da), r2 * sin(angle + da), -width * 0.5f);
			ffp.ffpNormal3f(cos(angle), sin(angle), 0.0f);
			ffp.ffpVertex3f(r2 * cos(angle + 2 * da), r2 * sin(angle + 2 * da), width * 0.5f);
			ffp.ffpVertex3f(r2 * cos(angle + 2 * da), r2 * sin(angle + 2 * da), -width * 0.5f);
			u = r1 * cos(angle + 3 * da) - r2 * cos(angle + 2 * da);
			v = r1 * sin(angle + 3 * da) - r2 * sin(angle + 2 * da);
			ffp.ffpNormal3f(v, -u, 0.0f);
			ffp.ffpVertex3f(r1 * cos(angle + 3 * da), r1 * sin(angle + 3 * da), width * 0.5f);
			ffp.ffpVertex3f(r1 * cos(angle + 3 * da), r1 * sin(angle + 3 * da), -width * 0.5f);
			ffp.ffpNormal3f(cos(angle), sin(angle), 0.0f);
		}

		ffp.ffpVertex3f(r1, 0.0f, width * 0.5f);
		ffp.ffpVertex3f(r1, 0.0f, -width * 0.5f);

		ffp.ffpEnd();

		ffp.ffpShadeModel(FFP_SMOOTH);

		/* draw inside radius cylinder */
		ffp.ffpBegin(FFP_QUAD_STRIP);
		for (i = 0; i <= teeth; i++) {
			angle = i * 2.0f * M_PI / teeth;
			ffp.ffpNormal3f(-cos(angle), -sin(angle), 0.0f);
			ffp.ffpVertex3f(r0 * cos(angle), r0 * sin(angle), -width * 0.5f);
			ffp.ffpVertex3f(r0 * cos(angle), r0 * sin(angle), width * 0.5f);
		}
		ffp.ffpEnd();
	}

}
