package ffp_examples;

/*
 *  idbuf.java
 *  Demonstrates use of the ID buffer.
 *  Click in the colored rectangle to set the sphere to that color.
 *
 */
public class idbuf extends DemoBase {

	public static void main(String[] args) {
		new idbuf().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		if (w <= h) {
			ffp.ffpOrtho(-1.5f, 1.5f, -1.5f * h / w, 1.5f * h / w, -10.0f, 10.0f);
		} else {
			ffp.ffpOrtho(-1.5f * w / h, 1.5f * w / h, -1.5f, 1.5f, -10.0f, 10.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	// ID 0 is reserved to represent a 'no-id' pixel
	static final int RED_ID = 1;
	static final int GREEN_ID = 2;
	static final int BLUE_ID = 3;
	static final int VIOLET_ID = 4;

	private static float red[] = { 0.5f, 0.0f, 0.0f };
	private static float grn[] = { 0.0f, 0.5f, 0.0f };
	private static float blu[] = { 0.0f, 0.0f, 0.5f };
	private static float vlt[] = { 0.5f, 0.0f, 0.5f };
	private static float cur[] = red;

	/*
	 *  Initialize material property, light source, lighting model,
	 *  and depth buffer.
	 */
	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_DEPTH_TEST);

		float mat_specular[] = { 1, 1, 1 };
		float light_position[] = { 1.0f, 1.0f, 1.0f, 0.0f };
		ffp.ffpMaterialfv(FFP_DIFFUSE, cur);
		ffp.ffpMaterialfv(FFP_SPECULAR, mat_specular);
		ffp.ffpMateriali(FFP_SHININESS, 25);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);
		ffp.ffpEnableLight(0);
		ffp.ffpSmoothModel(FFP_PHONG);

		ffp.ffpColorMaterial(FFP_DIFFUSE);
		ffp.ffpEnable(FFP_COLOR_MATERIAL);
	}

	@Override
	public void dbUpdate() {
		if (mouseLeftButtonPressed()) {
			switch (ffp.ffpGetIDAt(mouseX(), mouseY())) {
				case RED_ID:
					cur = red;
					break;
				case GREEN_ID:
					cur = grn;
					break;
				case BLUE_ID:
					cur = blu;
					break;
				case VIOLET_ID:
					cur = vlt;
					break;
				default:
					break;
			}
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER | FFP_ID_BUFFER);

		ffp.ffpDisable(FFP_LIGHTING);
		ffp.ffpIDWritemask(FFP_TRUE);

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-0.125f, 0.125f, 0);

		ffp.ffpIDVal(RED_ID);
		ffp.ffpColor3fv(red);
		ffp.ffpRectf(-1.25f, 1.25f, -0.75f, 0.75f);

		ffp.ffpIDVal(GREEN_ID);
		ffp.ffpColor3fv(grn);
		ffp.ffpRectf(-1.25f, 0.5f, -0.75f, 0.0f);

		ffp.ffpIDVal(BLUE_ID);
		ffp.ffpColor3fv(blu);
		ffp.ffpRectf(-1.25f, -0.25f, -0.75f, -0.75f);

		ffp.ffpIDVal(VIOLET_ID);
		ffp.ffpColor3fv(vlt);
		ffp.ffpRectf(-1.25f, -1f, -0.75f, -1.5f);
		ffp.ffpPopMatrix();

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpIDWritemask(FFP_FALSE);
		
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.25f, 0, 0);
		ffp.ffpColor3fv(cur);
		aux.auxSolidSphere(1.0f, 20, 16);
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
