package ffp_examples;

/*
 *  mandelbrot.java
 *  Draws the mandelbrot set
 */
public class mandelbrot extends DemoBase {

	public static void main(String[] args) {
		new mandelbrot().init(EUT_RGB | EUT_NOT_RESIZABLE);
	}

	static float xmax = 1.5f, xmin = -2.5f, ymax = 2.0f, ymin = -2.0f; //screen size
	static float intv = .001f; //pixels plotted
	static float rali, imagi; //Julia set initial points
	static boolean rendered = false;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection();
	}

	private static void create_projection() {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		ffp.ffpOrtho(xmin, xmax, ymin, ymax, -1.0f, 1.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0.0f, .1f, .1f);
		ffp.ffpShadeModel(FFP_FLAT);
	}

	@Override
	public void dbUpdate() {

	}

	static void color(int num) {
		int temp;
		temp = num % 10;
		switch (temp) {
			case 1:
				ffp.ffpColor3f(0, .1f, .1f);
				break;
			case 2:
				ffp.ffpColor3f(0, .2f, .1f);
				break;
			case 3:
				ffp.ffpColor3f(0, .3f, .1f);
				break;
			case 4:
				ffp.ffpColor3f(0, .4f, .1f);
				break;
			case 5:
				ffp.ffpColor3f(0, .5f, .1f);
				break;
			case 6:
				ffp.ffpColor3f(0, .6f, .1f);
				break;
			case 7:
				ffp.ffpColor3f(0, .7f, .1f);
				break;
			case 8:
				ffp.ffpColor3f(0, .8f, .1f);
				break;
			case 9:
				ffp.ffpColor3f(0, .9f, .1f);
				break;
			case 0:
				ffp.ffpColor3f(0, 1, 1);
				break;
			default:
				break;
		}
	}

	@Override
	public void dbRender() {
		if (!rendered) {
			ffp.ffpClear(FFP_VIDEO_BUFFER);
			ffp.ffpColor3f(1.0f, 0.0f, 0.0f);
			ffp.ffpBegin(FFP_POINTS);
			float imag, ral, temp;
			float i, t;
			float max, min;
			float check;
			int iter;

			min = -3.0f; //presets for max and min
			max = 3.0f;

			for (i = min; i < max; i += intv)
				for (t = min; t < max; t += intv) {
					check = 0;
					iter = 0;
					imag = i;
					ral = t;
					rali = t;
					imagi = i;
					while ((check < 4) && (iter < 100)) {
						//plots the points for the equation
						temp = ral;
						ral = (ral * ral) - (imag * imag) + rali;
						imag = (imagi) + (2 * temp * imag);
						check = ((ral * ral) + (imag * imag));
						iter++;
					}
					//checks for bounding 
					if (check < 4) {
						ffp.ffpColor3f(0.0f, 0.0f, 0.0f);
					}
					if (check > 4) {
						color(iter);
					}
					ffp.ffpVertex2f(t, i);
				}
			ffp.ffpEnd();
			ffp.ffpFlush(FFP_MULTI_THREADED);
			rendered = true;
		}
	}

}
