package ffp_examples;

/*
 *  movelight.java
 *  This program demonstrates when to issue lighting and
 *  transformation commands to render a model with a light
 *  which is moved by a modeling transformation (rotate or
 *  translate).  The light position is reset after the modeling
 *  transformation is called.  The eye position does not change.
 *
 *  A sphere is drawn using a grey material characteristic.
 *  A single light source illuminates the object.
 *
 *  Interaction:  holding the left mouse button alters
 *  the modeling transformation (x rotation) by 1 degree.
 *  The scene is then redrawn with the light in a new position.
 */
public class movelight extends DemoBase {

	public static void main(String[] args) {
		new movelight().init(EUT_RGB);
	}

	static float position[] = { 0.0f, 0.0f, 1.5f, 1.0f };
	static int spin = 0;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(40.0f, (float) w / (float) h, 1.0f, 20.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		float ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
		float diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
		float position[] = { 0.0f, 3.0f, 2.0f, 0.0f };
		int lmodel_ambient[] = { 34, 34, 34 };

		ffp.ffpLightfv(0, FFP_AMBIENT, ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, diffuse);
		ffp.ffpLightfv(0, FFP_POSITION, position);
		ffp.ffpLightModeliv(FFP_LIGHT_MODEL_AMBIENT, lmodel_ambient);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_LOCAL_VIEWER, FFP_FALSE);

	}

	@Override
	public void dbUpdate() {
		if (mouseLeftButtonActive()) {
			spin = (spin + 1) % 360;
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		aux.auxLookAt(0.0f, 0.0f, 5.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);

		ffp.ffpPushMatrix();
		ffp.ffpRotatef(spin, 1.0f, 0.0f, 0.0f);
		ffp.ffpLightfv(0, FFP_POSITION, position);

		ffp.ffpTranslatef(0.0f, 0.0f, 1.5f);
		ffp.ffpDisable(FFP_LIGHTING);
		ffp.ffpColor3ub(0, 255, 255);
		aux.auxWireCube(0.1f);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpPopMatrix();

		aux.auxSolidTorus(0.275f, 0.85f, 8, 15);
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
