package ffp_examples;

/*
 *  quad.java
 *  A much more advanced display of the quadric functions.
 */
public class quad extends DemoBase {

	static final int RD = 0xa40000;
	static final int WT = 0xffffff;

	static float position[] = { 1.0f, 1.0f, 1.0f, 0.0f };

	static float diffuse1[] = { 0.75f, 0.0f, 0.0f };
	static float position1[] = { -1.0f, -1.0f, -1.0f, 0.0f };

	static float front_mat_shininess[] = { 100.0f };
	static float front_mat_specular[] = { 1f, 1f, 1f };
	static float front_mat_diffuse[] = { 0.3f, 0.48f, 0.9f };

	static boolean dither = true;
	static boolean shade = true;
	static boolean texture = true;
	static int draw_type = FFPAUX_FILL;
	static int orient = FFPAUX_OUTSIDE;
	static int shade_type = FFPAUX_SMOOTH;
	static int tex_param = FFP_TRUE;

	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 4;
	static int xold, yold;
	static int radius1, radius2;
	static float angle1, angle2;
	static int slices, stacks;
	static int cheight;
	static int whichQuadric = 0;

	static int brickImageWidth = 16;
	static int brickImageHeight = 16;

	static int[] brickImage = { // ..
			RD, RD, RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, // start
			RD, RD, RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, // ..
			RD, RD, RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, // ..
			RD, RD, RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, // ..
			WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, // ..
			RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, // ..
			RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, // ..
			RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, // ..
			RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, // ..
			WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, // ..
			RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, RD, RD, // ..
			RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, RD, RD, // ..
			RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, RD, RD, // ..
			RD, RD, RD, RD, RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, RD, RD, // ..
			WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, WT, // end
			RD, RD, RD, RD, WT, RD, RD, RD, RD, RD, RD, RD, RD, RD, WT, RD };

	public static void main(String[] args) {
		new quad().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection();
	}

	private static void create_projection() {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		ffp.ffpFrustum(-1, 1, -1, 1, 1, 10);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);

		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpPointSize(5);
		ffp.ffpLightfv(0, FFP_POSITION, position);
		ffp.ffpLightfv(1, FFP_DIFFUSE, diffuse1);
		ffp.ffpLightfv(1, FFP_POSITION, position1);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_TWO_SIDE, FFP_TRUE);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnableLight(1);

		ffp.ffpMaterialfv(FFP_SHININESS, front_mat_shininess);
		ffp.ffpMaterialfv(FFP_SPECULAR, front_mat_specular);
		ffp.ffpMaterialfv(FFP_DIFFUSE, front_mat_diffuse);

		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_NEAREST);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_MODULATE);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, brickImage, brickImageWidth, brickImageHeight);
		radius1 = 10;
		radius2 = 5;
		angle1 = 90;
		angle2 = 180;
		slices = 16;
		stacks = 10;
		cheight = 20;
	}

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}
		xold = x;
		yold = y;
		if (keyPressed(KEY_1)) {
			draw_type = FFPAUX_FILL;
		}
		if (keyPressed(KEY_2)) {
			draw_type = FFPAUX_LINE;
		}
		if (keyPressed(KEY_3)) {
			draw_type = FFPAUX_SILHOUETTE;
		}
		if (keyPressed(KEY_4)) {
			draw_type = FFPAUX_POINT;
		}
		if (keyPressed(KEY_5)) {
			shade = !shade;
			if (shade) {
				ffp.ffpShadeModel(FFP_SMOOTH);
				shade_type = FFPAUX_SMOOTH;
			} else {
				ffp.ffpShadeModel(FFP_FLAT);
				shade_type = FFPAUX_FLAT;
			}
		}
		if (keyPressed(KEY_6)) {
			if (ffp.ffpGetInteger(FFP_SMOOTH_MODEL) == FFP_PHONG) {
				ffp.ffpSmoothModel(FFP_GOURAUD);
			} else {
				ffp.ffpSmoothModel(FFP_PHONG);
			}
		}
		if (keyPressed(KEY_A) && keyHeld(KEY_SHIFT)) {
			stacks++;
		}
		if (keyPressed(KEY_A) && !keyHeld(KEY_SHIFT)) {
			stacks--;
		}
		if (keyPressed(KEY_S) && keyHeld(KEY_SHIFT)) {
			slices++;
		}
		if (keyPressed(KEY_S) && !keyHeld(KEY_SHIFT)) {
			slices--;
		}
		if (keyPressed(KEY_D)) {
			switch (orient) {
				case FFPAUX_OUTSIDE:
					orient = FFPAUX_INSIDE;
					break;
				default:
					orient = FFPAUX_OUTSIDE;
					break;
			}
		}

		if (keyPressed(KEY_F)) {
			whichQuadric = (whichQuadric + 1) & 3;
		}
		if (keyPressed(KEY_G) && keyHeld(KEY_SHIFT)) {
			radius1++;
		}
		if (keyPressed(KEY_G) && !keyHeld(KEY_SHIFT)) {
			radius1--;
		}
		if (keyPressed(KEY_J) && keyHeld(KEY_SHIFT)) {
			radius2++;
		}
		if (keyPressed(KEY_J) && !keyHeld(KEY_SHIFT)) {
			radius2--;
		}
		if (keyPressed(KEY_H) && keyHeld(KEY_SHIFT)) {
			cheight += 2;
		}
		if (keyPressed(KEY_H) && !keyHeld(KEY_SHIFT)) {
			cheight -= 2;
		}
		if (keyPressed(KEY_K) && keyHeld(KEY_SHIFT)) {
			angle1 += 5;
		}
		if (keyPressed(KEY_K) && !keyHeld(KEY_SHIFT)) {
			angle1 -= 5;
		}
		if (keyPressed(KEY_L) && keyHeld(KEY_SHIFT)) {
			angle2 += 5;
		}
		if (keyPressed(KEY_L) && !keyHeld(KEY_SHIFT)) {
			angle2 -= 5;
		}
		if (keyPressed(KEY_Z)) {
			texture = !texture;
			if (texture) {
				tex_param = FFPAUX_TRUE;
				ffp.ffpEnable(FFP_TEXTURE_2D);
			} else {
				tex_param = FFPAUX_FALSE;
				ffp.ffpDisable(FFP_TEXTURE_2D);
			}
		}
		if (keyPressed(KEY_Q)) {
			ffp.ffpDisable(FFP_CULL_FACE);
		}
		if (keyPressed(KEY_W)) {
			ffp.ffpEnable(FFP_CULL_FACE);
			ffp.ffpCullFace(FFP_FRONT);
		}
		if (keyPressed(KEY_E)) {
			ffp.ffpEnable(FFP_CULL_FACE);
			ffp.ffpCullFace(FFP_BACK);
		}
		if (keyPressed(KEY_Y)) {
			dither = !dither;
			if (dither) {
				ffp.ffpVideoPropertyi(FFP_VIDEO_COLOR_DITHER, FFP_TRUE);
				ffp.ffpColormapCreate(3, 3, 3);
			} else {
				ffp.ffpVideoPropertyi(FFP_VIDEO_COLOR_DITHER, FFP_FALSE);
			}
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -translate_z);
		ffp.ffpRotatef(rotate_y, 1, 0, 0);
		ffp.ffpRotatef(rotate_x, 0, 1, 0);

		switch (draw_type) {
			case FFPAUX_SILHOUETTE:
				ffp.ffpColor3f(1.0f, 1.0f, 0.0f);
				break;
			case FFPAUX_LINE:
				ffp.ffpColor3f(0.0f, 1.0f, 1.0f);
				break;
			default:
				ffp.ffpColor3f(1.0f, 1.0f, 1.0f);
				break;
		}
		switch (whichQuadric) {
			case 0:
				ffp.ffpTranslatef(0, 0, -cheight / 20.0f);
				aux.auxCylinder(radius1 / 10.0f, radius2 / 10.0f, cheight / 10.0f, slices, stacks, draw_type, orient, shade_type, tex_param);
				break;
			case 1:
				aux.auxSphere(radius1 / 10.0f, slices, stacks, draw_type, orient, shade_type, tex_param);
				break;
			case 2:
				aux.auxPartialDisk(radius2 / 10.0f, radius1 / 10.0f, slices, stacks, angle1, angle2, draw_type, orient, shade_type, tex_param);
				break;
			case 3:
				aux.auxDisk(radius2 / 10.0f, radius1 / 10.0f, slices, stacks, draw_type, orient, shade_type, tex_param);
				break;
			default:
				break;
		}
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
