package ffp_examples;

/*
 * sierpinski.java
 * This program demonstrates the fractal sierpinski triangle created
 * by randomly choosing between three vertices and moving the point halfway
 * to the chosen vertex.
 */
public class sierpinski extends DemoBase {

	public static void main(String[] args) {
		new sierpinski().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		if (w <= h) {
			aux.auxOrtho2D(0.0f, 2.0f, 0.0f, 2.0f * h / w);
		} else {
			aux.auxOrtho2D(0.0f, 2.0f * w / h, 0.0f, 2.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		point[0] = random() * 2;
		point[1] = random() * 2;
	}

	@Override
	public void dbUpdate() {

	}

	private float[] point = new float[2];

	private float[][] verts = { { 1.75f, 0.25f }, { 1.0f, 1.75f }, { 0.25f, 0.25f } };

	@Override
	public void dbRender() {
		ffp.ffpPointSize(4);
		ffp.ffpBegin(FFP_POINTS);
		ffp.ffpColor3f(0.0f, 1.0f, 0.0f);
		ffp.ffpVertex2fv(verts[0]);
		ffp.ffpVertex2fv(verts[1]);
		ffp.ffpVertex2fv(verts[2]);
		ffp.ffpEnd();
		for (int i = 0; i < 32; i++) {
			int vert = (int) (random() * 3);
			point[0] = (verts[vert][0] + point[0]) / 2.0f;
			point[1] = (verts[vert][1] + point[1]) / 2.0f;
			ffp.ffpPointSize(1);
			ffp.ffpBegin(FFP_POINTS);
			ffp.ffpColor3f(point[0] / 2.0f, ((point[0] + point[1]) / 2) / 2.0f, point[1] / 2.0f);
			ffp.ffpVertex2fv(point);
			ffp.ffpEnd();
		}

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
