package ffp_examples;

/*
 *  steam.java
 */
public class steam extends DemoBase {

	public static void main(String[] args) {
		new steam().init(EUT_RGB);
	}

	static final float M_PI = 3.14159265f;

	static final int IMAGE_WIDTH = 64;
	static final int IMAGE_HEIGHT = 64;
	static final int ANGLE_STEP = 10;
	static final float MAGNITUDE = 120;
	static final float PHASE = 270.112f;
	static final float FREQ_DIV = 58;
	static final float ARC_LENGTH = 2.7f;
	static final float ARC_RADIUS = 0.15f;

	static float head_angle = 0;
	static int crank_angle = 0;
	static float crank_step = 5;

	static boolean shaded = true;
	static boolean texture = true;
	static boolean light1 = true;
	static boolean light2 = false;

	static int quad_norm = FFPAUX_SMOOTH;
	static int quad_draw = FFPAUX_FILL;
	static int quad_text = FFPAUX_TRUE;

	static float[] head_lut = new float[361];
	static int[] image = new int[IMAGE_WIDTH * IMAGE_HEIGHT];

	static float mat_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float mat_shininess[] = { 50.0f };
	static float light_position1[] = { 1.0f, 1.0f, 1.0f, 0.0f };
	static float light_position2[] = { -1.0f, 1.0f, 1.0f, 0.0f };
	static float light_diffuse2[] = { 0.21f, 0.21f, 0.61f };

	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 6;
	static int xold, yold;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(65.0f, (float) w / (float) h, 1.0f, 20.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -5.0f);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		make_table();
		make_image();

		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_NEAREST);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_BLOCK_FILTER, FFP_TRUE);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_MODULATE);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, image, IMAGE_WIDTH, IMAGE_HEIGHT);

		ffp.ffpMaterialfv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialfv(FFP_SHININESS, mat_shininess);
		ffp.ffpLightfv(0, FFP_POSITION, light_position1);
		ffp.ffpLightfv(1, FFP_DIFFUSE, light_diffuse2);
		ffp.ffpLightfv(1, FFP_POSITION, light_position2);

		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_NORMALIZE);
		ffp.ffpColorMaterial(FFP_DIFFUSE);
		ffp.ffpEnable(FFP_COLOR_MATERIAL);
		quad_text = FFPAUX_FALSE;
	}

	static void make_image() {
		int i, j;

		for (i = 0; i < IMAGE_WIDTH; i++) {
			for (j = 0; j < IMAGE_HEIGHT; j++) {
				if ((((i & 0x8) == 0) ^ ((j & 0x8)) == 0)) {
					image[i + j * IMAGE_WIDTH] = 0xffffff;
				} else {
					image[i + j * IMAGE_WIDTH] = 0;
				}
			}
		}
	}

	static void make_table() {
		int i;
		float k;

		for (i = 0, k = 0.0f; i < 360; i++, k++) {
			head_lut[i] = MAGNITUDE * atan((ARC_RADIUS * sin(PHASE - k / FREQ_DIV)) / ((ARC_LENGTH - ARC_RADIUS * cos(PHASE - k / FREQ_DIV))));
		}
	}

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}
		xold = x;
		yold = y;
		if (keyPressed(KEY_S)) {
			if (!shaded) {
				shaded = true;
				ffp.ffpShadeModel(FFP_SMOOTH);
				ffp.ffpEnable(FFP_LIGHTING);
				ffp.ffpEnable(FFP_DEPTH_TEST);
				ffp.ffpEnable(FFP_COLOR_MATERIAL);
				quad_norm = FFPAUX_SMOOTH;
				quad_draw = FFPAUX_FILL;
			} else {
				shaded = false;
				ffp.ffpShadeModel(FFP_FLAT);
				ffp.ffpDisable(FFP_LIGHTING);
				ffp.ffpDisable(FFP_DEPTH_TEST);
				ffp.ffpDisable(FFP_COLOR_MATERIAL);
				quad_norm = FFPAUX_NONE;
				quad_draw = FFPAUX_LINE;
				quad_text = FFPAUX_FALSE;
			}
		}

		if (keyPressed(KEY_T)) {
			if (!texture) {
				texture = true;
				ffp.ffpEnable(FFP_TEXTURE_2D);
				quad_text = FFPAUX_TRUE;
			} else {
				texture = false;
				ffp.ffpDisable(FFP_TEXTURE_2D);
				quad_text = FFPAUX_FALSE;
			}
		}

		if (keyHeld(KEY_A)) {
			if ((crank_angle += crank_step) >= 360)
				crank_angle = 0;
			head_angle = head_lut[crank_angle];
		}
		if (keyHeld(KEY_Z)) {
			if ((crank_angle -= crank_step) <= 0)
				crank_angle = 360;
			head_angle = head_lut[crank_angle];
		}
		if (keyPressed(KEY_1)) {
			if (light1) {
				ffp.ffpDisableLight(0);
				light1 = false;
			} else {
				ffp.ffpEnableLight(0);
				light1 = true;
			}
		}
		if (keyPressed(KEY_2)) {
			if (light2) {
				ffp.ffpDisableLight(1);
				light2 = false;
			} else {
				ffp.ffpEnableLight(1);
				light2 = true;
			}
		}

		if (keyHeld(KEY_V)) {
			if ((++crank_step) > 45)
				crank_step = 45;
		}
		if (keyHeld(KEY_B)) {
			if ((--crank_step) <= 0)
				crank_step = 0;
		}
		if ((crank_angle += crank_step) >= 360) {
			crank_angle = 0;
		}
		head_angle = head_lut[crank_angle];
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -translate_z);
		ffp.ffpRotatef(rotate_y, 1, 0, 0);
		ffp.ffpRotatef(rotate_x, 0, 1, 0);

		draw_engine_pole();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.5f, 1.4f, 0.0f);
		draw_cylinder_head();
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.0f, -0.8f, 0.0f);
		draw_crank();
		ffp.ffpPopMatrix();
		ffp.ffpPopMatrix();

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

	static void box(float x, float y, float z) {
		ffp.ffpPushMatrix();
		ffp.ffpScalef(x, y, z);
		if (shaded) {
			aux.auxSolidCube(1);
		} else {
			aux.auxWireCube(1);
		}
		ffp.ffpPopMatrix();
	}

	static void closed_cylinder(float outerRadius, float innerRadius, float length) {
		ffp.ffpPushMatrix();
		aux.auxCylinder(outerRadius, outerRadius, length, 20, 1, quad_draw, FFPAUX_OUTSIDE, quad_norm, quad_text);
		ffp.ffpPushMatrix();
		ffp.ffpRotatef(180, 0, 1, 0);
		aux.auxDisk(innerRadius, outerRadius, 20, 1, quad_draw, FFPAUX_OUTSIDE, quad_norm, quad_text);
		ffp.ffpPopMatrix();
		ffp.ffpTranslatef(0, 0, length);
		aux.auxDisk(innerRadius, outerRadius, 20, 1, quad_draw, FFPAUX_OUTSIDE, quad_norm, quad_text);
		ffp.ffpPopMatrix();
	}

	private static void draw_piston() {
		ffp.ffpPushMatrix();
		ffp.ffpColor3f(0.07f, 0.33f, 0.8f);
		ffp.ffpPushMatrix();
		ffp.ffpRotatef(90, 0, 1, 0);
		ffp.ffpTranslatef(0, 0, -0.07f);
		closed_cylinder(0.125f, 0.06f, 0.12f);
		ffp.ffpPopMatrix();
		ffp.ffpRotatef(-90, 1, 0, 0);
		ffp.ffpTranslatef(0, 0, 0.05f);
		closed_cylinder(0.06f, 0.0f, 0.6f);
		ffp.ffpTranslatef(0, 0, 0.6f);
		closed_cylinder(0.2f, 0.0f, 0.5f);
		ffp.ffpPopMatrix();
	}

	private static void draw_engine_pole() {
		ffp.ffpPushMatrix();
		ffp.ffpColor3f(0.8f, 0.8f, 0.8f);
		box(0.5f, 3.0f, 0.5f);
		ffp.ffpColor3f(0.21f, 0.1f, 0.21f);
		ffp.ffpRotatef(90, 0, 1.0f, 0);
		ffp.ffpTranslatef(0.0f, 0.9f, -0.4f);
		closed_cylinder(0.1f, 0.0f, 2);
		ffp.ffpPopMatrix();
	}

	private static void draw_cylinder_head() {
		ffp.ffpPushMatrix();
		ffp.ffpColor3f(0.21f, 1.0f, 0.21f);
		ffp.ffpRotatef(90, 1.0f, 0.0f, 0.0f);
		ffp.ffpTranslatef(0, 0.0f, 0.4f);
		ffp.ffpRotatef(head_angle, 1, 0, 0);
		ffp.ffpTranslatef(0, 0.0f, -0.4f);
		closed_cylinder(0.23f, 0.21f, 1.6f);
		ffp.ffpRotatef(180, 1.0f, 0.0f, 0.0f);
		aux.auxDisk(0, 0.23f, 20, 1, quad_draw, FFPAUX_OUTSIDE, quad_norm, quad_text);
		ffp.ffpPopMatrix();
	}

	private static void draw_flywheel() {
		ffp.ffpPushMatrix();
		ffp.ffpColor3f(0.21f, 0.21f, 1.0f);
		ffp.ffpRotatef(90, 0.0f, 1.0f, 0.0f);
		closed_cylinder(0.625f, 0.08f, 0.5f);
		ffp.ffpPopMatrix();
	}

	private static void draw_crankbell() {
		ffp.ffpPushMatrix();
		ffp.ffpColor3f(1.0f, 0.21f, 0.21f);
		ffp.ffpRotatef(90f, 0.0f, 1.0f, 0.0f);
		closed_cylinder(0.3f, 0.08f, 0.12f);

		ffp.ffpColor3f(0.21f, 0.1f, 0.21f);
		ffp.ffpTranslatef(0.0f, 0.2f, 0.0f);
		closed_cylinder(0.06f, 0.0f, 0.34f);

		ffp.ffpTranslatef(0.0f, 0.0f, 0.22f);
		ffp.ffpRotatef(90, 0.0f, 1.0f, 0.0f);
		ffp.ffpRotatef(crank_angle - head_angle, 1.0f, 0.0f, 0.0f);
		draw_piston();
		ffp.ffpPopMatrix();
	}

	private static void draw_crank() {
		ffp.ffpPushMatrix();
		ffp.ffpRotatef(crank_angle, 1.0f, 0.0f, 0.0f);

		ffp.ffpPushMatrix();
		ffp.ffpRotatef(90, 0.0f, 1.0f, 0.0f);
		ffp.ffpTranslatef(0.0f, 0.0f, -1.0f);
		closed_cylinder(0.08f, 0.0f, 1.4f);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.28f, 0.0f, 0.0f);
		draw_crankbell();
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-0.77f, 0.0f, 0.0f);
		draw_flywheel();
		ffp.ffpPopMatrix();
		ffp.ffpPopMatrix();
	}

}
