package ffp_examples;

/*
 *  watersurf.java
 */
public class watersurf extends DemoBase {

	static final int RES = 12;
	static final float DELTA = 2.0f / RES;
	static final int GRID_SIZE = 4;

	static float[] verts = new float[6 * RES * (RES + 1)];
	static float[] norms = new float[6 * RES * (RES + 1)];
	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 4;

	static float ctx = 0;
	static float cty = 0;

	static float amp = 0.01f;
	static float freq = 20.0f;
	static float t_freq = 4.0f;

	static float time;
	static int xold, yold;

	static float light_ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	static float light_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float light_position[] = { 0.0f, 2.5f, 0.0f, 1.0f };

	static int tex_width = 32;
	static int tex_height = 32;
	static int[] tex = new int[tex_width * tex_height];

	static int dtex_width = 32;
	static int dtex_height = 32;
	static int[] dtex = new int[dtex_width * dtex_height];

	static boolean draw_normals = false;
	static boolean phong = false;
	static boolean wire_frame = false;
	static boolean stip = true;

	static short[] pattern = { //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55 };

	static void gen_texture() {
		for (int i = 0; i < tex_width; i++) {
			for (int j = 0; j < tex_height; j++) {
				int idx = i + j * tex_width;
				int r = (int) (pow((sqrt(rand(25, 8192))) / 255.0, 2.2) * 255);
				int g = (int) (pow(rand(160, 220) / 255.0, 2.2) * 255);
				int b = (int) (pow((237 + rand(0, 8)) / 255.0, 2.2) * 255);
				tex[idx] = r << 16 | g << 8 | b;
				tex[idx] = color_blend(tex[idx], 0xffffff, (int) (random() * 128));
			}
		}
		for (int i = 0; i < dtex_width; i++) {
			for (int j = 0; j < dtex_height; j++) {
				int idx = i + j * dtex_width;
				int b = (int) (pow((sqrt(rand(25, 8192))) / 255.0, 2.2) * 255);
				int g = (int) (pow(rand(160, 220) / 255.0, 2.2) * 255);
				int r = (int) (pow((237 + rand(0, 8)) / 255.0, 2.2) * 255);
				dtex[idx] = r << 16 | g << 8 | b;
				dtex[idx] = color_blend(dtex[idx], 0xffffff, 192 + (int) (random() * 63));
				if (random() > 0.98) {
					dtex[idx] = color_blend(dtex[idx], 0, (int) (random() * 63));
				}
			}
		}
	}

	static double rand(int min, int max) {
		int n = abs(max - min);
		return min(min, max) + (n == 0 ? 0 : random() * n);
	}

	static int color_blend(int c1, int c2, int factor) {
		int f1 = 255 ^ factor;
		return ((((c1 & 0xff00ff) * f1 + (c2 & 0xff00ff) * factor) & 0xff00ff00) | (((c1 & 0x00ff00) * f1 + (c2 & 0x00ff00) * factor) & 0x00ff0000)) >> 8;
	}

	static float z(float x, float y, float t) {
		float xx = x * x;
		float yy = y * y;
		return (sin(freq * sqrt(xx + yy) - t_freq * t) * noise.get_noise(10 * x, 10 * y, t, 0)) * amp;
	}

	public static void main(String[] args) {
		new watersurf().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(45, w / (float) h, 0.1f, 150f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		noise.noise_init();
		gen_texture();
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_BILINEAR);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_BLOCK_FILTER, FFP_FALSE);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_MODULATE);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_SPHERE_MAP, FFP_TRUE);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, tex, tex_width, tex_height);
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 1);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_DITHER);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_BLOCK_FILTER, FFP_FALSE);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_MODULATE);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, dtex, dtex_width, dtex_height);
		ffp.ffpLightfv(0, FFP_AMBIENT, light_ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, light_diffuse);
		ffp.ffpLightfv(0, FFP_SPECULAR, light_specular);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);
		ffp.ffpMaterialfv(FFP_SPECULAR, light_specular);
		ffp.ffpMateriali(FFP_SHININESS, 127);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpLightModelfv(FFP_LIGHT_MODEL_AMBIENT, new float[] { 0.5f, 0.5f, 0.5f });
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_LOCAL_VIEWER, FFP_TRUE);
		ffp.ffpEnable(FFP_CULL_FACE);
		ffp.ffpPolygonStipple(pattern);
		ffp.ffpVideoPropertyi(FFP_VIDEO_BLEND_STIPPLED_REGIONS, FFP_TRUE);
	}

	@Override
	public void dbUpdate() {
		time += 0.01f;
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}
		if (keyPressed(KEY_N)) {
			draw_normals = !draw_normals;
		}
		if (keyPressed(KEY_W)) {
			wire_frame = !wire_frame;
		}
		if (keyPressed(KEY_Q)) {
			stip = !stip;
			ffp.ffpVideoPropertyi(FFP_VIDEO_BLEND_STIPPLED_REGIONS, stip ? FFP_TRUE : FFP_FALSE);
		}
		if (keyPressed(KEY_S)) {
			phong = !phong;
			ffp.ffpSmoothModel(phong ? FFP_PHONG : FFP_GOURAUD);
		}
		xold = x;
		yold = y;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpEnable(FFP_POLYGON_STIPPLE);
		ffp.ffpEnable(FFP_TEXTURE_2D);
		ffp.ffpEnable(FFP_LIGHTING);
		int i;
		int j;
		float x;
		float y;
		int idx;
		int preidx;

		float v1x;
		float v1y;
		float v1z;

		float v2x;
		float v2y;
		float v2z;

		float v3x;
		float v3y;
		float v3z;

		float vax;
		float vay;
		float vaz;

		float vbx;
		float vby;
		float vbz;

		float nx;
		float ny;
		float nz;

		float l;

		/* Render wireframe? */
		if (wire_frame) {
			ffp.ffpPolygonMode(FFP_FRONT_AND_BACK, FFP_LINE);
		}
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);

		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -translate_z);
		ffp.ffpRotatef(rotate_y, 1, 0, 0);
		ffp.ffpRotatef(rotate_x, 0, 1, 0);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);

		for (int u = -GRID_SIZE; u <= GRID_SIZE; u++) {
			for (int v = -GRID_SIZE; v <= GRID_SIZE; v++) {
				ctx = u * 2f;
				cty = v * 2f;
				/* Vertices */
				for (j = 0; j < RES; j++) {
					y = cty + (j + 1) * DELTA - 1;
					for (i = 0; i <= RES; i++) {
						idx = 6 * (i + j * (RES + 1));

						x = ctx + i * DELTA - 1;
						verts[idx + 3] = x;
						verts[idx + 4] = z(x, y, time);
						verts[idx + 5] = y;
						if (j != 0) {
							/* Values were computed during the previous loop */
							preidx = 6 * (i + (j - 1) * (RES + 1));
							verts[idx] = verts[preidx + 3];
							verts[idx + 1] = verts[preidx + 4];
							verts[idx + 2] = verts[preidx + 5];
						} else {
							verts[idx] = x;
							verts[idx + 1] = z(x, -1 + cty, time);
							verts[idx + 2] = -1 + cty;
						}
					}
				}

				/* Normals */
				for (j = 0; j < RES; j++) {
					for (i = 0; i <= RES; i++) {
						idx = 6 * (i + j * (RES + 1));
						v1x = verts[idx + 3];
						v1y = verts[idx + 4];
						v1z = verts[idx + 5];

						v2x = v1x;
						v2y = verts[idx + 1];
						v2z = verts[idx + 2];

						if (i < RES) {
							v3x = verts[idx + 9];
							v3y = verts[idx + 10];
							v3z = v1z;
						} else {
							float xn = ctx + (RES + 1) * DELTA + 1;
							v3x = xn;
							v3y = z(xn, v1z, time);
							v3z = v1z;
						}

						vax = v2x - v1x;
						vay = v2y - v1y;
						vaz = v2z - v1z;

						vbx = v3x - v1x;
						vby = v3y - v1y;
						vbz = v3z - v1z;

						nx = (vby * vaz) - (vbz * vay);
						ny = (vbz * vax) - (vbx * vaz);
						nz = (vbx * vay) - (vby * vax);

						l = sqrt(nx * nx + ny * ny + nz * nz);
						if (l != 0) {
							l = 1 / l;
							norms[idx + 3] = nx * l;
							norms[idx + 4] = ny * l;
							norms[idx + 5] = nz * l;
						} else {
							norms[idx + 3] = 0;
							norms[idx + 4] = 1;
							norms[idx + 5] = 0;
						}

						if (j != 0) {
							/* Values were computed during the previous loop */
							preidx = 6 * (i + (j - 1) * (RES + 1));
							norms[idx] = norms[preidx + 3];
							norms[idx + 1] = norms[preidx + 4];
							norms[idx + 2] = norms[preidx + 5];
						} else {
							v1y = z(v1x, (j - 1) * DELTA - 1, time);
							v1z = cty + (j - 1) * DELTA - 1;

							v3y = z(v3x, v2z, time);
							v3z = v2z;

							vax = v1x - v2x;
							vay = v1y - v2y;
							vaz = v1z - v2z;

							vbx = v3x - v2x;
							vby = v3y - v2y;
							vbz = v3z - v2z;

							nx = (vby * vaz) - (vbz * vay);
							ny = (vbz * vax) - (vbx * vaz);
							nz = (vbx * vay) - (vby * vax);

							l = sqrt(nx * nx + ny * ny + nz * nz);
							if (l != 0) {
								l = 1 / l;
								norms[idx] = nx * l;
								norms[idx + 1] = ny * l;
								norms[idx + 2] = nz * l;
							} else {
								norms[idx] = 0;
								norms[idx + 1] = 1;
								norms[idx + 2] = 0;
							}
						}
					}
				}
				draw_water_patch();
			}
		}

		ffp.ffpDisable(FFP_POLYGON_STIPPLE);
		ffp.ffpDisable(FFP_LIGHTING);
		ffp.ffpPushMatrix();
		ffp.ffpColor3ub(0x88, 0x88, 0x44);
		ffp.ffpTranslatef(0, -1, 0);
		ffp.ffpRotatef(-90, 1, 0, 0);
		float tsc = 10;
		ffp.ffpPolygonMode(FFP_FRONT_AND_BACK, FFP_FILL);
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 1);
		ffp.ffpBegin(FFP_QUADS);
		ffp.ffpTexCoord2f(0, 0);
		ffp.ffpVertex2f(-9, -9);
		ffp.ffpTexCoord2f(tsc, 0);
		ffp.ffpVertex2f(9, -9);
		ffp.ffpTexCoord2f(tsc, tsc);
		ffp.ffpVertex2f(9, 9);
		ffp.ffpTexCoord2f(0, tsc);
		ffp.ffpVertex2f(-9, 9);
		ffp.ffpEnd();
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

	public static void draw_water_patch() {
		ffp.ffpEnable(FFP_TEXTURE_2D);
		ffp.ffpBegin(FFP_TRIANGLE_STRIP);
		int xx = 0;
		int yy = 0;
		for (int i = 0; i < verts.length - 2; i += 3) {
			if ((yy & 1) == 0) {
				ffp.ffpNormal3f(norms[i], norms[i + 1], norms[i + 2]);
				ffp.ffpVertex3f(verts[i], verts[i + 1], verts[i + 2]);
				if (xx == RES) {
					xx = 0;
					yy++;
					ffp.ffpNormal3f(norms[i + 3], norms[i + 4], norms[i + 5]);
					ffp.ffpVertex3f(verts[i + 3], verts[i + 4], verts[i + 5]);
					ffp.ffpEnd();
					ffp.ffpBegin(FFP_TRIANGLE_STRIP);
					ffp.ffpNormal3f(norms[i], norms[i + 1], norms[i + 2]);
					ffp.ffpVertex3f(verts[i], verts[i + 1], verts[i + 2]);
				} else {
					xx++;
				}
				continue;
			}
			ffp.ffpNormal3f(norms[i], norms[i + 1], norms[i + 2]);
			ffp.ffpVertex3f(verts[i], verts[i + 1], verts[i + 2]);
			if (xx == RES) {
				xx = 0;
				yy++;
				ffp.ffpEnd();
				ffp.ffpBegin(FFP_TRIANGLE_STRIP);
			} else {
				xx++;
			}
		}
		ffp.ffpEnd();
		if (draw_normals) {
			ffp.ffpDisable(FFP_TEXTURE_2D);
			ffp.ffpColor3f(1, 0, 0);
			ffp.ffpBegin(FFP_LINES);
			for (int j = 0; j < RES; j++)
				for (int i = 0; i <= RES; i++) {
					int idx = 6 * (i + j * (RES + 1));
					ffp.ffpVertex3f(verts[idx], verts[idx + 1], verts[idx + 2]);
					ffp.ffpVertex3f(verts[idx] + norms[idx] / 10, verts[idx + 1] + norms[idx + 1] / 10, verts[idx + 2] + norms[idx + 2] / 10);
				}

			ffp.ffpEnd();
			ffp.ffpEnable(FFP_TEXTURE_2D);
		}
	}

	/**
	 * Perlin noise
	 */
	static class noise {

		static final int MOD = 0xff;

		static int[] permut = new int[256];
		static int[][] gradient = { //
				{ 1, 1, 1, 0 }, { 1, 1, 0, 1 }, //
				{ 1, 0, 1, 1 }, { 0, 1, 1, 1 }, //
				{ 1, 1, -1, 0 }, { 1, 1, 0, -1 }, //
				{ 1, 0, 1, -1 }, { 0, 1, 1, -1 }, //
				{ 1, -1, 1, 0 }, { 1, -1, 0, 1 }, //
				{ 1, 0, -1, 1 }, { 0, 1, -1, 1 }, //
				{ 1, -1, -1, 0 }, { 1, -1, 0, -1 }, //
				{ 1, 0, -1, -1 }, { 0, 1, -1, -1 }, //
				{ -1, 1, 1, 0 }, { -1, 1, 0, 1 }, //
				{ -1, 0, 1, 1 }, { 0, -1, 1, 1 }, //
				{ -1, 1, -1, 0 }, { -1, 1, 0, -1 }, //
				{ -1, 0, 1, -1 }, { 0, -1, 1, -1 }, //
				{ -1, -1, 1, 0 }, { -1, -1, 0, 1 }, //
				{ -1, 0, -1, 1 }, { 0, -1, -1, 1 }, //
				{ -1, -1, -1, 0 }, { -1, -1, 0, -1 }, //
				{ -1, 0, -1, -1 }, { 0, -1, -1, -1 }, //
		};

		static void noise_init() {
			int i = 0;
			while (i < 256) {
				permut[i++] = (int) (random() * MOD);
			}
		}

		/*
		** Function finding out the gradient corresponding to the coordinates
		*/
		static int index(int i, int j, int k, int l) {
			return (permut[(l + permut[(k + permut[(j + permut[i & MOD]) & MOD]) & MOD]) & MOD] & 0x1f);
		}

		/*
		** Functions computing the dot product of the vector and the gradient
		*/
		static float prod(float a, int b) {
			if (b > 0)
				return a;
			if (b < 0)
				return -a;
			return 0;
		}

		static float dot(float x1, int x2, float y1, int y2, float z1, int z2, float t1, int t2) {
			return (prod(x1, x2) + prod(y1, y2) + prod(z1, z2) + prod(t1, t2));
		}

		/*
		** Functions computing interpolations
		*/
		static float spline5(float state) {
			float sqr = state * state;
			return state * sqr * (6 * sqr - 15 * state + 10);
		}

		static float lerp(float start, float end, float state) {
			return start + (end - start) * state;
		}

		/*
		** Noise function, returning the Perlin Noise at a given point
		*/
		static float get_noise(float x, float y, float z, float t) {
			/* The unit hypercube containing the point */
			int x1 = (int) (x > 0 ? x : x - 1);
			int y1 = (int) (y > 0 ? y : y - 1);
			int z1 = (int) (z > 0 ? z : z - 1);
			int t1 = (int) (t > 0 ? t : t - 1);
			int x2 = x1 + 1;
			int y2 = y1 + 1;
			int z2 = z1 + 1;
			int t2 = t1 + 1;

			/* The 16 corresponding gradients */
			int[] g0000 = gradient[index(x1, y1, z1, t1)];
			int[] g0001 = gradient[index(x1, y1, z1, t2)];
			int[] g0010 = gradient[index(x1, y1, z2, t1)];
			int[] g0011 = gradient[index(x1, y1, z2, t2)];
			int[] g0100 = gradient[index(x1, y2, z1, t1)];
			int[] g0101 = gradient[index(x1, y2, z1, t2)];
			int[] g0110 = gradient[index(x1, y2, z2, t1)];
			int[] g0111 = gradient[index(x1, y2, z2, t2)];
			int[] g1000 = gradient[index(x2, y1, z1, t1)];
			int[] g1001 = gradient[index(x2, y1, z1, t2)];
			int[] g1010 = gradient[index(x2, y1, z2, t1)];
			int[] g1011 = gradient[index(x2, y1, z2, t2)];
			int[] g1100 = gradient[index(x2, y2, z1, t1)];
			int[] g1101 = gradient[index(x2, y2, z1, t2)];
			int[] g1110 = gradient[index(x2, y2, z2, t1)];
			int[] g1111 = gradient[index(x2, y2, z2, t2)];

			/* The 16 vectors */
			float dx1 = x - x1;
			float dx2 = x - x2;
			float dy1 = y - y1;
			float dy2 = y - y2;
			float dz1 = z - z1;
			float dz2 = z - z2;
			float dt1 = t - t1;
			float dt2 = t - t2;

			/* The 16 dot products */
			float b0000 = dot(dx1, g0000[0], dy1, g0000[1], dz1, g0000[2], dt1, g0000[3]);
			float b0001 = dot(dx1, g0001[0], dy1, g0001[1], dz1, g0001[2], dt2, g0001[3]);
			float b0010 = dot(dx1, g0010[0], dy1, g0010[1], dz2, g0010[2], dt1, g0010[3]);
			float b0011 = dot(dx1, g0011[0], dy1, g0011[1], dz2, g0011[2], dt2, g0011[3]);
			float b0100 = dot(dx1, g0100[0], dy2, g0100[1], dz1, g0100[2], dt1, g0100[3]);
			float b0101 = dot(dx1, g0101[0], dy2, g0101[1], dz1, g0101[2], dt2, g0101[3]);
			float b0110 = dot(dx1, g0110[0], dy2, g0110[1], dz2, g0110[2], dt1, g0110[3]);
			float b0111 = dot(dx1, g0111[0], dy2, g0111[1], dz2, g0111[2], dt2, g0111[3]);
			float b1000 = dot(dx2, g1000[0], dy1, g1000[1], dz1, g1000[2], dt1, g1000[3]);
			float b1001 = dot(dx2, g1001[0], dy1, g1001[1], dz1, g1001[2], dt2, g1001[3]);
			float b1010 = dot(dx2, g1010[0], dy1, g1010[1], dz2, g1010[2], dt1, g1010[3]);
			float b1011 = dot(dx2, g1011[0], dy1, g1011[1], dz2, g1011[2], dt2, g1011[3]);
			float b1100 = dot(dx2, g1100[0], dy2, g1100[1], dz1, g1100[2], dt1, g1100[3]);
			float b1101 = dot(dx2, g1101[0], dy2, g1101[1], dz1, g1101[2], dt2, g1101[3]);
			float b1110 = dot(dx2, g1110[0], dy2, g1110[1], dz2, g1110[2], dt1, g1110[3]);
			float b1111 = dot(dx2, g1111[0], dy2, g1111[1], dz2, g1111[2], dt2, g1111[3]);

			/* Then the interpolations, down to the result */
			float idx1 = spline5(dx1);
			float idy1 = spline5(dy1);
			float idz1 = spline5(dz1);
			float idt1 = spline5(dt1);

			float b111 = lerp(b1110, b1111, idt1);
			float b110 = lerp(b1100, b1101, idt1);
			float b101 = lerp(b1010, b1011, idt1);
			float b100 = lerp(b1000, b1001, idt1);
			float b011 = lerp(b0110, b0111, idt1);
			float b010 = lerp(b0100, b0101, idt1);
			float b001 = lerp(b0010, b0011, idt1);
			float b000 = lerp(b0000, b0001, idt1);

			float b11 = lerp(b110, b111, idz1);
			float b10 = lerp(b100, b101, idz1);
			float b01 = lerp(b010, b011, idz1);
			float b00 = lerp(b000, b001, idz1);

			float b1 = lerp(b10, b11, idy1);
			float b0 = lerp(b00, b01, idy1);

			return lerp(b0, b1, idx1);
		}

	}

}
